#!/bin/sh
#------------------------------------------------------------------------------
# Asterisk - fli4l Web Interface
#   Copyright (C) 2008-2010 Tiger
#
#   This OPT package is licensed under Free Software Foundation, Inc.,
#   General Public License Version 2, June 1991
#------------------------------------------------------------------------------

. /srv/www/include/cgi-helper
. /var/run/asterisk.conf

# Program Globals
asteriskerrorlevel=0

errorlevel()
{
  asteriskerrorlevel=$1
}

check_UserRight()
{
  local required_right=$1
  # ------------------------
  result=""
  if [ -n "$REMOTE_USER" ]
  then
    local rights="`grep \"^$REMOTE_USER=\" /etc/httpd/rights | cut -d= -f2`"
    local right
    local localrights=
    local globalrights=
    for right in $rights
    do
      case "`echo $right | sed -n 's|^\([^:]*\):.*$|\1|p'`" in
        "$ASTERISK_RIGHTS_SECTION")
          localrights="`echo $right | sed 's|.*:||'`"
          if [ -z "$localrights" ]; then localrights="none"; fi
          ;;
        "") globalrights=$right;;
      esac
    done
    rights=$localrights
    if [ -z "$rights" ]; then rights=$globalrights; fi
    if [ -z "$rights" ]; then rights="all"; fi
    for right in `echo $rights | tr ',' ' '`
    do
      case $right in
        all) result="yes";;
        $required_right) result="yes";;
      esac
    done
  fi
  if [ -n "$result" ]
  then
    echo "yes"
  else
    errorlevel 1
  fi
}

for_all_channeltypes()
{
  local command=$1
  local command_total=$2
  # ------------------------
  local count=0
  local total=0
  local listed=
  local type=
  local description=
  $ASTERISK_FILE_RCOMM "core show channeltypes" | filterANSI | sed -n '3,$ s|^\(.\{11\}\) \(.\{40\}\) \(.\{12\}\) \(.\{12\}\) \(.\{8\}\)|\1 \2|p' |
  (
    while read type description
    do
      case $type in
        SIP|IAX|IAX2)
          eval $command
          if [ -n "$command_total" ]; then eval $command_total; fi
          listed=yes
        ;;
      esac
    done
    if [ -z "$listed" -a -z "$command_total" ]; then show "<tr><td><b>$_asterisk_status_nodata</b></td></tr>"; fi
    if [ $count -gt 0 ]; then echo "$count $_asterisk_of $total"; fi
  )
}

get_LocalizedTimeInfo()
{
  local timeinfo=$*
  # ------------------------
  set $_asterisk_timestring
  echo $timeinfo | sed "s| years| $1|; \
                        s| year| $2|; \
                        s| weeks| $3|; \
                        s| week| $4|; \
                        s| days| $5|; \
                        s| day| $6|; \
                        s| hours| $7|; \
                        s| hour| $8|; \
                        s| minutes| $9|; \
                        s| minute| $10|; \
                        s| seconds| $11|; \
                        s| second| $12|"
}

filterANSI()
{
  sed 's|\[[0-9;]*m||g;\|^$|d'
}

ANSItoHTML()
{
  sed 's|\[0m|</span>|g;
       s|\[[0-9];30[;0-9]*m|<span style="color: black;">|g;
       s|\[[0-9];31[;0-9]*m|<span style="color: red;">|g;
       s|\[[0-9];32[;0-9]*m|<span style="color: green;">|g;
       s|\[[0-9];33[;0-9]*m|<span style="color: maroon;">|g;
       s|\[[0-9];34[;0-9]*m|<span style="color: blue;">|g;
       s|\[[0-9];35[;0-9]*m|<span style="color: fuchsia;">|g;
       s|\[[0-9];36[;0-9]*m|<span style="color: aqua;">|g;
       s|\[[0-9];37[;0-9]*m|<span style="color: white;">|g;
       s|\[[0-9;]*m||g;\|^$|d'
}

show()
{
  local text=$*
  # ------------------------
  echo "$text"
}

show_Overview()
{
  local command=$1
  # ------------------------
  case $command in
    reload) $ASTERISK_FILE_RCOMM "module reload" > /dev/null; sleep 3;;
  esac
  show "<table class=\"normtable\">"
    show "<tr id=asterisk_status><th><a href=\"$ASTERISK_HOMEPAGE\" target=\"_blank\" alt=\"OPT_Asterisk\"><image src=\"../img/opt-asterisk-logo.png\"></a></th><td>`$ASTERISK_FILE_EXE -V | sed 's|-| |g'`</td></tr>"
    show "<tr><th>&nbsp $_asterisk_header_uptime &nbsp</th><td>`get_LocalizedTimeInfo $($ASTERISK_FILE_RCOMM \"core show uptime\" | filterANSI | sed -n 's|System uptime: \(.*\)|\1|p')`</td></tr>"
    local reload="`$ASTERISK_FILE_RCOMM \"core show uptime\" | filterANSI | sed -n 's|Last reload: \(.*\)|\1|p'`"
    if [ -z "$reload" ]; then reload="-"; else reload=`get_LocalizedTimeInfo $reload`; fi
    show "<tr><th>&nbsp $_asterisk_header_lastreload &nbsp</th><td>$reload</td></tr>"
    local regs=$(for_all_channeltypes 'count=`expr $count + $($ASTERISK_FILE_RCOMM "$type show registry" | filterANSI | grep -c " Registered ")`' 'total=`expr $total + $($ASTERISK_FILE_RCOMM "$type show registry" | filterANSI | grep -c "") - 2`')
    if [ `echo $regs | sed "s|$_asterisk_of|=|"` ]; then class=asterisk_pass; else class=asterisk_fail; fi
    if [ -z "$regs" ]; then regs="$_asterisk_none"; fi
    show "<tr><th>&nbsp $_asterisk_header_registry &nbsp</th><td class=$class>$regs</td></tr>"
    local peers=$(for_all_channeltypes 'count=`expr $count + $($ASTERISK_FILE_RCOMM "$type show peers" | filterANSI | grep -c -v " UNKNOWN ") - 2`' 'total=`expr $total + $($ASTERISK_FILE_RCOMM "$type show peers" | filterANSI | grep -c "") - 2`')
    if [ `echo $peers | sed "s|$_asterisk_of|=|"` ]; then class=asterisk_pass; else class=asterisk_fail; fi
    if [ -z "$peers" ]; then peers="$_asterisk_none"; fi
    show "<tr><th>&nbsp $_asterisk_header_peers &nbsp</th><td class=$class>$peers</td></tr>"
    local chans=$(for_all_channeltypes 'count=`expr $count + $($ASTERISK_FILE_RCOMM "$type show channels" | filterANSI | grep -c -v " 0x0 (nothing) ") - 2`' 'total=`expr $total + $($ASTERISK_FILE_RCOMM "$type show channels" | filterANSI | grep -c "") - 2`')
    if [ -z "$chans" ]; then chans="$_asterisk_none"; fi
    show "<tr><th>&nbsp $_asterisk_header_channels &nbsp</th><td>$chans</td></tr>"
    if [ -n "`check_UserRight admin`" ]
    then
      show "<form action=\"$myname\" method=\"GET\">"
        show "<input type=\"hidden\" name=\"action\" value=\"overview\">"
        show "<input type=\"hidden\" name=\"cmd\" value=\"reload\">"
        show "<tr><td colspan=2 align=\"center\">&nbsp<input class=\"sbutton\" type=\"submit\" value=\"$_asterisk_button_reload\">&nbsp</td></tr>"
      show "</form>"
    fi
  show "</table>"
  show "<br>"
}

show_Registry()
{
  local channel=$1
  local description=$2
  # ------------------------
  show "`show_tab_header \"$description\" no`"
    show "<table class=\"normtable\">"
      show "<tr>"
        show "<th>$_asterisk_header_host</th>"
        show "<th>$_asterisk_header_user</th>"
        show "<th>$_asterisk_header_status</th>"
        show "<th>$_asterisk_header_regtime</th>"
      show "</tr>"
      local line=
      $ASTERISK_FILE_RCOMM "$channel show registry" | filterANSI |
        sed -n '2,$ s|^\([^ ]*\) [ ]*[A-Z]* [ ]*\([^ ]*\) [ ]*\([0-9]*\) \([No ]*[^ ]*\) [ ]*\([0-9a-zA-Z, :]*\)$|<tr><td>\1</td><td>\2</td>(STATE:\4:\5)</td></tr>|p' |
        sed 's|(STATE:Registered:\(.*\))|<td class=asterisk_pass align=center>'"$_asterisk_status_registered"'</td><td>\1|;\
          s|(STATE:Unregistered:.*)|<td class=asterisk_fail align=center>'"$_asterisk_status_unregistered"'</td><td>-|;\
          s|(STATE:No Authentication:.*)|<td class=asterisk_fail align=center>'"$_asterisk_status_noauthentication"'</td><td>-|;\
          s|(STATE:\(.*\):.*)|<td align=center>\1</td><td>-|' |
        sort |
        while read line; do show "$line"; [ 1 = 2 ]; done && show "<tr><td colspan=4><b>$_asterisk_status_nodata</b></td></tr>"
    show "</table>"
  show "`show_tab_footer`"
}

show_Peers()
{
  local channel=$1
  local description=$2
  # ------------------------
  show "`show_tab_header \"$description\" no`"
    show "<table class=\"normtable\">"
      show "<tr>"
        show "<th>$_asterisk_header_extension</th>"
        show "<th>$_asterisk_header_ipandport</th>"
        show "<th>$_asterisk_header_status</th>"
        show "<th>$_asterisk_header_delay</th>"
      show "</tr>"
      local line=
      $ASTERISK_FILE_RCOMM "$channel show peers" | filterANSI |
        sed -n '2,$ s|^\([^ ]*\) [ ]*\((.*)\|[0-9.]\{7,17\}\)[ ]* [(S) ]*[0-9.]*[DNA ]* \([0-9]*\)[ ]* [(E) ]*\(.*[^ ]\)[ ]*$|<tr><td>\1</td>(HOST:\2:\3)</td>(STATUS:\4)</td></tr>|p' |
        sed 's|(HOST:(Unspecified):0)|<td class=asterisk_fail align=center>'"$_asterisk_status_offline"'|;\
          s|(HOST:\([^)]*\))|<td>\1|;\
          s|(STATUS:OK (\([0-9]*\) ms))|<td class=asterisk_pass align=center>'"$_asterisk_status_online"'</td><td align=center>\1|;\
          s|(STATUS:UNKNOWN)|<td class=asterisk_fail align=center>'"$_asterisk_status_unknown"'</td><td align=center>-|;\
          s|(STATUS:Unmonitored)|<td align=center>'"$_asterisk_status_unmonitored"'</td><td align=center>-|;\
          s|(STATUS:UNREACHABLE)|<td class=asterisk_fail align=center>'"$_asterisk_status_unreachable"'</td><td align=center>-|;\
          s|(STATUS:\(.*\))|<td align=center>\1</td><td align=center>-|' |
        sort |
        while read line; do show "$line"; [ 1 = 2 ]; done && show "<tr><td colspan=4><b>$_asterisk_status_nodata</b></td></tr>"
    show "</table>"
  show "`show_tab_footer`"
}

show_Channels()
{
  local channel=$1
  local description=$2
  # ------------------------
  show "`show_tab_header \"$description\" no`"
    show "<table class=\"normtable\">"
      show "<tr>"
        show "<th>$_asterisk_header_extension</th>"
        show "<th>$_asterisk_header_peer</th>"
        show "<th>$_asterisk_header_format</th>"
        show "<th>$_asterisk_header_parked</th>"
        show "<th>$_asterisk_header_lastmessage</th>"
      show "</tr>"
      local line=
      $ASTERISK_FILE_RCOMM "$channel show channels" | filterANSI |
        sed -n '2,$\
          s|^\([0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*\)[ ]* \([^ ]*\) [ ]*\([^ ]*\) [ ]*\(0x[0-9a-fA-F]* ([^ ]*\)[)|] [ ]*\([YesNo]*\) [ ]*\([RT]x: [^ ]*\) .*$|<tr><td>\2</td><td>\1</td><td align="center">\4)</td><td align=\"center\">(HOLD:\5)</td><td>\6</td></tr>|p' |
        sed '/(HOLD:Yes)/ s|^<tr>|<tr class=asterisk_fail>|;\
          /(HOLD:No)/ {/0x0 (nothing)/ !{s|^<tr>|<tr class=asterisk_pass>|}}' |
        sed 's|(HOLD:No)|'$_asterisk_no'|;s|(HOLD:Yes)|'$_asterisk_yes'|p' |
        while read line; do show "$line"; [ 1 = 2 ]; done && show "<tr><td colspan=5><b>$_asterisk_status_nodata</b></td></tr>"
    show "</table>"
  show "`show_tab_footer`"
}

show_TableRegExFilter()
{
  local name=$1
  local value=$2
  # ------------------------
  show "<th><input class=\"itext\" type=\"text\" size=\"20\" maxlength=\"80\" name=\"$name\" value=\"`echo \"$value\" | sed 's|\\\\(||g;s|\\\\)||g'`\"></th>"
}

show_CallFile()
{
  local sortfield=$1
  local sortorder=$2
  local filterfrom=$3
  local filterto=$4
  local filtercaller=$5
  local filtercallername=$6
  local filtercallee=$7
  local filterline=$8
  local filteruserfield=$9
  # ------------------------
  if [ -f $ASTERISK_FILE_CALLS ]
  then
    show "<table class=\"normtable\">"
      show "<form action=\"$myname\" method=\"GET\">"
        show "<tr>"
          show "<td colspan=7><h2><a href=\"$myname?download=callfile\">$ASTERISK_FILE_CALLS</a></h2></td>"
          show "<td colspan=1 align=\"right\">&nbsp<input class=\"sbutton\" type=\"submit\" value=\"$_MN_show\">&nbsp</td>"
        show "</tr>"
        show "<tr>"
          local sortparam=""
          local firstrow=
          case $sortfield in
            caller)      firstrow='\1';;
            callee)      firstrow='\2';;
            callername)  firstrow='\3';;
            line)        firstrow='\4';;
            duration)    firstrow='\6'; sortparam="-n";;
            disposition) firstrow='\7';;
            userfield)   firstrow='\8';;
            *)           firstrow='\5'; sortfield=datetime;;
          esac
          local field=
          local header=
          local image=
          local slink=
          for field in disposition datetime caller callername callee line duration userfield
          do
            eval header="\$_asterisk_header_$field"
            if [ "$field" = "$sortfield" ]
            then
              if [ "$sortorder" = "rev" ]
              then
                image="<img src=\"../img/snormal.gif\">"; slink=""; sortparam="$sortparam -r"
              else
                image="<img src=\"../img/sreverse.gif\">"; slink="&srdr=rev"
              fi
            else
              image=""; slink=""
            fi
            header="<a href=\"$myname?action=calls&sfld=$field$slink&sfrom=`url_encode $filterfrom`&sto=`url_encode $filterto`&scaller=`url_encode $filtercaller`&scallee=`url_encode $filtercallee`&sname=`url_encode $filtercallername`&sline=`url_encode $filterline`&sufld=`url_encode $filteruserfield`\">$header$image</a>"
            show "<th>$header</th>"
          done
        show "</tr>"
        show "<tr>"
          show "<th>"
          show "<input type=\"hidden\" name=\"action\" value=\"calls\">"
          show "<input type=\"hidden\" name=\"sfld\" value=\"$sortfield\">"
          show "<input type=\"hidden\" name=\"srdr\" value=\"$sortorder\">"
          show "</th>"
          show "<th>"
            show "<input class=\"itext\" type=\"text\" size=\"10\" maxlength=\"20\" name=\"sfrom\" value=\"$filterfrom\"> - "
            show "<input class=\"itext\" type=\"text\" size=\"10\" maxlength=\"20\" name=\"sto\" value=\"$filterto\">"
          show "</th>"
          show_TableRegExFilter scaller "$filtercaller"
          show_TableRegExFilter sname "$filtercallername"
          show_TableRegExFilter scallee "$filtercallee"
          show_TableRegExFilter sline "$filterline"
          show "<th></th>"
          show_TableRegExFilter sufld "$filteruserfield"
        show "</tr>"
      show "</form>"
      local reccount=0
      local durcount=0
      local filter=
      local value=
      for filter in filtercaller filtercallername filtercallee filterline filteruserfield
      do
        eval value="\$$filter"
        eval $filter="`echo \"$value\" | sed 's|\\\\(||g;s|\\\\)||g;s|\.\*|[^\\\\\"]*|g;/^$/ s||[^\\\\\"]*|'`"
      done
      local line=
         # s|^"[^"]*",\("[^"]*"\),"[^"]*","[^"]*",\("[^"]*","[^"]*","[^"]*"\),"[^"]*",\("[^\|,"]*\)[^"]*",\("[^"]*"\),[^,]*,"[^"]*",[0-9]*,\([0-9]*,"[^"]*"\),"[^"]*",\("[^"]*"\).*|\1,\3",\2,\4,\5,\6|;
      cat $ASTERISK_FILE_CALLS | sed 's|,"""\([^"]*\)""[^"]*",|,"\1",|;\
          \|^[^,]*,[^,]*,"[s]*",| s|^\([^,]*,[^,]*\),[^,]*,"\([^"]*\)",\([^,]*,[^,]*,[^,]*,[^,]*\),"\([^\|,"]*\)[^"]*",\(.*\)$|\1,"\2(\4)","\2",\3,"\4",\5|;\
          s|^"[^"]*",\("[^"]*","[^"]*"\),"[^"]*",\("[^"]*","[^"]*","[^"]*"\),"[^"]*","[^"]*",\("[^"]*"\),[^,]*,"[^"]*",[0-9]*,\([0-9]*,"[^"]*"\),"[^"]*",\("[^"]*"\).*|\1,\2,\3,\4,\5|;\
          s|^\([^,]*,[^,]*,[^,]*\),"\([^"]*\)","",\(.*\)|\1,"\2","\2",\3|;\
          s|^\([^,]*,[^,]*,[^,]*,[^,]*\),"\([^"]*\)-[0-9a-zA-Z]*",\(.*\)|\1,"\2",\3|' |
        sed -n '/}"$/ s|\(.*\),\([^,]*\),\([^,]*\){\([^{}]*\)}"$|\1,"\4",\3"|;\
          s|.*|&,""|;\
          s|^"\('"$filtercaller"'\)","\('"$filtercallee"'\)","\('"$filtercallername"'\)","[^"]*","\('"$filterline"'\)","\([^"]*\)",\([0-9]*\),"\([^,]*\)","\('"$filteruserfield"'\)".*|'$firstrow';cdate="\5";duration="\6";line="(DISPOSITION:\7)</td><td>\5</td><td>\1</td><td>\3</td><td>\2</td><td>\4</td><td>(DURATION:\6)</td><td>\8</td>"|p' |
        sort $sortparam |
        sed 's|^[^;]*;||;\
          s|(DISPOSITION:4)|(DISPOSITION:ANSWERED)|;\
          s|(DISPOSITION:1)|(DISPOSITION:NO ANSWER)|;\
          s|(DISPOSITION:2)|(DISPOSITION:BUSY)|;\
          s|(DISPOSITION:3)|(DISPOSITION:FAILED)|;\
          s|(DISPOSITION:ANSWERED)|<td class=asterisk_pass>$_asterisk_status_answered|;\
          s|(DISPOSITION:NO ANSWER)|<td class=asterisk_fail>$_asterisk_status_noanswer|;\
          s|(DISPOSITION:BUSY)|<td class=asterisk_fail>$_asterisk_status_busy|;\
          s|(DISPOSITION:FAILED)|<td class=asterisk_fail>$_asterisk_status_failed|;\
          s|(DISPOSITION:\([^)]*\))|<td class=asterisk_fail>\1|' |
        (
          while read line
          do
            eval $line
            if [ -z "$filterfrom" -o `expr "$cdate" \< "$filterfrom"` = 0 ]
            then
              if [ -z "$filterto" -o `expr "$cdate" \>= "$filterto"` = 0 ]
              then
                reccount=`expr $reccount + 1`
                durcount=`expr $durcount + 0$duration`
                echo "$line" | sed 's|(DURATION:\([0-9]*\))|'$($ASTERISK_FILE_S2HMS 0$duration)'|'
              fi
            fi
          done &&
          if [ 0$reccount -gt 0 ]
          then
            echo "<tr>\
<td align=\"right\"><b>$reccount</b></td>\
<th>&nbsp $_asterisk_header_reccounter &nbsp</th>\
<th colspan=3></th>\
<th align=\"right\">&nbsp $_asterisk_header_durcounter &nbsp</th>\
<td><b>`$ASTERISK_FILE_S2HMS 0$durcount`</b></td>\
<th></th>\
</tr>"
          fi
        ) |
        sed '$!N;\
          /<tr>/ !{s|\(.*\)\n\(.*\)|<tr>\1</tr>\n<tr class=asterisk_linecoloring>\2</tr>|}' |
        while read line; do show "$line"; [ 1 = 2 ]; done && show "<tr><td colspan=8><b>$_asterisk_status_nodata</b></td></tr>"
    show "</table>"
  else
    show_error "$ASTERISK_PACKAGE" "$_asterisk_error_cdrfilenotfound ($ASTERISK_FILE_CALLS)"
  fi
}

show_AstDB()
{
  local family=$1
  local key=$2
  local data=$3
  local command=$4
  # ------------------------
  if [ -n "$family" ]; then key="`echo \"$family/$key\" | sed 's|//|/|'`"; fi
  local c_family="`echo \"$key\" | sed 's|^[/]*||' | sed -n 's|^\(.*\)/.*|\1|p'`"
  local c_key="`echo \"$key\" | sed -n 's|^.*/\(.*\)|\1|p'`"
  local c_data=
  local bgcolor=
  if [ -n "`echo \"$c_family/$c_key\" | sed -n 's| : |&|p'`" ]; then c_key=""; fi
  if [ -n "$c_family" -a -n "$c_key" -a `expr length "/$c_family/$c_key"` -le 50 ]
  then
    case "$command" in
      "kill")
        $ASTERISK_FILE_RCOMM "database del \"$c_family\" \"$c_key\"" > /dev/null
        if [ 0`$ASTERISK_FILE_RCOMM 'database show "'$c_family'"' | sed -n 's|^\([0-9]*\) results found.|\1|p'` -gt 0 ]; then key="/$c_family/"; else key="/"; fi
        c_key="";c_data=""
      ;;
      "add"|"update")
        $ASTERISK_FILE_RCOMM "database put \"$c_family\" \"$c_key\" \"$data\"" > /dev/null
        key="/$c_family/";c_key="";c_data=""
      ;;
      *)
        key="/$c_family/$c_key"
        c_data="$data"
      ;;
    esac
    bgcolor=""
  else
    case "$command" in
      "add") bgcolor="class=\"asterisk_badinput\"";;
      *) bgcolor="";;
    esac
  fi
  show "<table class=\"normtable\">"
    show "<tr><td colspan=7><h2><a href=\"$myname?download=astdb\">$ASTERISK_FILE_ASTDB</a></h2></td></tr>"
    show "<tr><th>$_asterisk_header_key</th><th>$_asterisk_header_value</th><th></th></tr>"
    if [ -z "$key" ]; then key="/"; fi
    local prev_key=""
    local linecoloring=""
    local s_key="`echo \"$key\" | sed 's|/|\\\/|g;s|\&|\\\&|g;s|\*|\\\*|g;s|\.|\\\.|g;s|\[|\\\[|g;s|\]|\\\]|g;s|\^|\\\^|g'`"
    local s_keyroot="`echo \"$key\" | sed 's|[^/]*$||;h;s|^/\([^/]*\).*|/\1/|;s|/|\\\/|g;x;s|^/[^/]*/\(.*\)$|\1|;s|[^/]*/|[^\\\/]*[\\\/]*|g;x;G;s|\n||'`"
    local line=
    # sed:
    #   Entries starting with current selection only
    #     Print if current selection is a property ..
    #     .. else ..
    #       Print if current selection contains a property ..
    #       .. else it's a subgroup, so add a exclamation mark to make it stick at the top of the list during sort
    #   Entries not starting with current selection: List root group only
    #   Delete everything else
    # echo: Add a line for each subgroup of the current selection, as to guarantee, there is at least one matching entry in the list. This is needed for browsing through subgroups.
    # sort: case-insensive and ignore any special characters
    # sed:  Remove any preciding exclamation mark in subgroup names of the current selection, as they were added for sorting only (see above)
    # sed
    #   Delete duplicate lines. Now we're dealing with the content of the current selection and the roots of all other groups, only.
    # sed
    #   Duplicate line, strip data, replace irrelevant subgroup names with undeletable white space and store white space and property name in separate lines
    #   Line #1: /root/subgroup/property : data
    #   Line #2: wsp wsp
    #   Line #3: property
    # sed
    #   Duplicate first line, strip tailing blanks, put url-encoded group and data in separate lines
    #   Line #1: /root/subgroup/property : data
    #   Line #2: wsp wsp
    #   Line #3: property
    #   Line #4: url-encoded group
    #   Line #5: url-encoded data
    # sed
    #   If first line contains a data delimiter (therefore data), HTML-encode entire set of lines as property
    # sed
    #   If line isn't allready HTML-encoded ..
    #     If property is a subgroup of the current selection, HTML-encode entire set of lines as subgroup
    #     Else HTML-encode set of lines as subgroup pointing to the current selection
    # sed: Insert line coloring to every second line
    $ASTERISK_FILE_RCOMM "database show" | filterANSI | ( \
      sed '\
        /^'"$s_key"'/ {\
          /^'"$s_key"'[ ]* : / P;\
          /^'"$s_key"'[ ]* : / !{\
            /^'"$s_key"'[^\/]* : / P;\
            /^'"$s_key"'[^\/]* : / !{s|^'"$s_key"'\([^/]*/\).*|'"$s_key"'!\1|p}}};\
        /^'"$s_key"'/ !{s|^\(/[^/]*[/]\).*|\1|p};\
        D' && \
      echo -n "$key" | sed 's|/\(.*\)/[^/]*|\1|;s|/|\n|g' | sed -n -e '/^$/ !{x;G;s|\n|/|p;x}' | sed 's|.*|&/|' ) |
      sort -f -d |
      sed 's|^'"$s_key"'!\([^/]*/\).*|'"$s_key"'\1|p' |
      sed '$!N;\
        /^\(.*\)\n\1$/!P;\
        D' |
      sed 'h;\
        s|[ ]* : .*||;\
        s|\([^/]*\)[/]*$|\n\1|;\
        s|^/||;\
        s|[^/]*/|\&nbsp\&nbsp\&nbsp\&nbsp\&nbsp|g;\
        x;G' |
      sed 'h;\
        s|[ ]*$||;s|[ ]* : |\n|;\
        s/%/%25/g;s/ /%20/g;s/!/%21/g;s/"/%22/g;s/#/%23/g;s/\$/%24/g;s/\&/%26/g;s/'\''/%27/g;s/(/%28/g;s/)/%29/g;s/\*/%2a/g;s/+/%2b/g;s/,/%2c/g;s/-/%2d/g;s/\./%2e/g;s/\//%2f/g;s/:/%3a/g;s/;/%3b/g;s/</%3c/g;s/\=/%3d/g;s/>/%3e/g;s/?/%3f/g;s/@/%40/g;s/\[/%5b/g;s/\\/%5c/g;s/\]/%5d/g;s/\^/%5e/g;s/_/%5f/g;s/`/%60/g;s/{/%7b/g;s/|/%7c/g;s/}/%7d/g;s/~/%7e/g;s//%f6/g;s//%fc/g;s//%e4/g;s//%d6/g;s//%dc/g;s//%c4/g;\
        x;N;N;G' |
      sed '\
        / : / !{N;N;N}\
        / : / {N;N;N;N;s|\(.*[^ ]\) [ ]*: \(.*[^ ]\).*\n\(.*\)\n\(.*\)\n\(.*\)\n\(.*\)|<td>\3<a href="'"$myname"'?action=astdb\&key=\5\&data=\6"><img src=\"../img/asterisk-key.gif\">\4</a></td><td>\2</td><td><a href="'"$myname"'?action=astdb\&key=\5\&cmd=kill"><img src=\"../img/asterisk-erase.gif\"></a></td></tr>|}' |
      sed '/^<td>/ !{\
          /^'"$s_keyroot"'$/ !{N;N;N;\
            s|\(.*[^ ]\).*\n\(.*\)\n\(.*\)\n\(.*\)|<td>\2<a href="'"$myname"'?action=astdb\&key=\4"><img src=\"../img/asterisk-plus.gif\">\3</a></td><td></td><td></td></tr>|}
          /^'"$s_keyroot"'$/ {N;N;N;\
              s|\(.*[^ ]\).*\n\(.*\)\n\(.*\)\n\(.*\)|<td>\2<a href="'"$myname"'?action=astdb\&key=\4"><b>\3</b><img src=\"../img/asterisk-arrow.gif\"></a></td><td></td><td></td></tr>|}}' |
      sed '$!N; s|\(.*\)\n\(.*\)|<tr>\1\n<tr class=asterisk_linecoloring>\2|' |
      while read line; do show "$line"; [ 1 = 2 ]; done && show "<tr><td colspan=5><b>$_asterisk_status_nodata</b></td></tr>"
    show "<form action=\"$myname\" method=\"POST\">"
      show "<tr><td $bgcolor>"
        show "<input type=\"hidden\" name=\"action\" value=\"astdb\">"
        show "<input type=\"hidden\" name=\"cmd\" value=\"add\">"
        show "<input class=\"itext\" type=\"text\" size=\"20\" maxlength=\"40\" name=\"family\" value=\"$c_family\">"
        show "<input class=\"itext\" type=\"text\" size=\"20\" maxlength=\"40\" name=\"key\" value=\"$c_key\">"
      show "</td>"
      show "<td><input class=\"itext\" type=\"text\" size=\"120\" maxlength=\"255\" name=\"data\" value=\"$c_data\"></td>"
      show "<td><input type=\"image\" style=\"background-color: transparent;border: transparent\" src=\"../img/asterisk-check.gif\"></td>"
    show "</tr></form>"
  show "</table>"
}

show_Config()
{
  local file=$1
  local content=$2
  # ------------------------
  local c_file="`echo \"$file\" | sed 's|[^a-zA-Z0-9._-]||g'`"
  local bgcolor=
  if [ "$c_file" = "$file" ]; then bgcolor=""; else file=""; bgcolor="class=\"asterisk_badinput\""; fi
  if [ -z "$file" -o -n "$content" ]
  then
    if [ -d $ASTERISK_DIR_CONF ]
    then
      if [ -n "$content" -a -n "$file" ]
      then
        echo "$content" | tr -d > $ASTERISK_DIR_CONF/$file
        show "<p><b>$_asterisk_status_fileupdated ($file)</b></p>"
        c_file=""
      fi
      show "<table class=\"normtable\">"
        show "<tr>"
          show "<th>$_asterisk_header_filename</th>"
          show "<th>$_asterisk_header_filesize</th>"
          show "<th>$_asterisk_header_filedate</th>"
        show "</tr>"
        local line=
        local filename=
        local filesize=
        local filedate=
        local link=
        ls -1let $ASTERISK_DIR_CONF | sed -n 's|\(.\{10\}\) \(.\{4\}\) \(.\{9\}\) \(.\{12\}\) \([^ ]*\)[ -> ]*\(.*\)|filename=\"\5\";filesize=\"\3\";filedate=\"\4\";link=\"\6\"|p' | sort | while read line
        do
          eval $line
          filesize=`echo $filesize`
          if [ -n "$link" ]
          then
            filesize=`ls -1let /$link | sed -n 's|\(.\{10\}\) \(.\{4\}\) \(.\{9\}\) \(.\{12\}\) .*|\3|p'`
            filedate=`ls -1let /$link | sed -n 's|\(.\{10\}\) \(.\{4\}\) \(.\{9\}\) \(.\{12\}\) .*|\4|p'`
          fi
          show "<tr>"
            show "<td>&nbsp<a href=\"$myname?action=config&file=$filename\">$filename</a>&nbsp<br></td>"
            show "<td align=\"right\">&nbsp $filesize &nbsp</td>"
            show "<td>&nbsp $filedate &nbsp</td>"
          show "</tr>"
          [ 1 = 2 ]
        done && show "<tr><td colspan=5><b>$_asterisk_status_nodata</b></td></tr>"
        show "<form action=\"$myname\" method=\"GET\">"
          show "<tr>"
            show "<input type=\"hidden\" name=\"action\" value=\"config\">"
            show "<td $bgcolor><input class=\"itext\" type=\"text\" size=\"25\" maxlength=\"40\" name=\"file\" value=\"$c_file\"></td>"
            show "<td colspan=2>&nbsp<input class=\"sbutton\" type=\"submit\" value=\"$_asterisk_button_newfile\">&nbsp</td>"
          show "</tr>"
        show "</form>"
      show "</table>"
    else
      show_error "$ASTERISK_PACKAGE" "$_asterisk_error_dirinaccessible ($ASTERISK_DIR_CONF)"
    fi
  else
    show "`show_tab_header \"$file\" no`"
    show "<table class=\"normtable\">"
      show "<form action=\"$myname\" method=\"POST\">"
        show "<input type=\"hidden\" name=\"action\" value=\"config\">"
        show "<input type=\"hidden\" name=\"file\" value=\"$file\">"
        show "<tr><td colspan=3><textarea name=\"content\" rows=\"40\" cols=\"100\">"
        if [ -f $ASTERISK_DIR_CONF/$file ]
        then
          show "`cat $ASTERISK_DIR_CONF/$file`"
        fi
        show "</textarea></td></tr>"
        show "<tr>"
          show "<td width=\"33%\" align=\"center\"><input class=\"sbutton\" type=\"button\" value=\"$_asterisk_button_abort\" onclick=\"document.location.href='$myname?action=config';return(false);\"></td>"
          show "<td width=\"33%\" align=\"center\"><input class=\"sbutton\" type=\"reset\"></td>"
          show "<td width=\"34%\" align=\"center\"><input class=\"sbutton\" type=\"submit\"></td>"
        show "</tr>"
      show "</form>"
    show "</table>"
    show "`show_tab_footer`"
  fi
}

show_CLI()
{
  local command=$1
  # ------------------------
  show "<table class=\"normtable\">"
    show "<tr>"
      show "<td colspan=2><textarea class=\"asterisk_cli\" name=\"content\" rows=\"40\" cols=\"80\">"
      if [ -n "$command" ]
      then
        show "`$ASTERISK_FILE_RCOMM \"$command\" | filterANSI`"
      fi
      show "</textarea></td>"
      show "<td rowspan=3 valign=\"bottom\">"
      show "<div id=\"asterisk_cli_buttons\">"
        local index=
        local cmd=
        local button_text=
        for index in `seq 1 0$ASTERISK_CLI_BTN_N`
        do
          eval cmd="\$ASTERISK_CLI_BTN_"$index
          if [ -z "`echo $cmd|sed -n 's|^---|&|p'`" ]
          then
            if [ `expr length "$cmd"` -le 23 ]
            then
              button_text=$cmd
            else
              button_text=`echo "$cmd"|sed -n 's|^\(.\{10\}\).*\(.\{10\}\)$|\1...\2|p'`
            fi
            show '<form action="'"$myname"'" method=POST>'
              show '<input type=hidden name=action value=cli>'
              show '<input type=hidden name=cmd value="'"$cmd"'">'
              show '<input class=sbutton type=submit value="'"$button_text"'">'
            show '</form>'
          else
            show "<hr>"
          fi
        done
      show "</div>"
      show "</td>"
    show "</tr>"
    show "<tr><td colspan=2>*CLI><a href=\"$myname?action=cli&cmd=`url_encode $command`\">$command</a></td></tr>"
    show "<form action=\"$myname\" method=\"POST\">"
      show "<tr>"
        show "<input type=\"hidden\" name=\"action\" value=\"cli\">"
        show "<td>*CLI><input class=\"itext\" type=\"text\" size=\"102\" maxlength=\"118\" name=\"cmd\"></td>"
        show "<td align=\"center\"><input type=\"image\" style=\"background-color: transparent;border: transparent\" src=\"../img/asterisk-check.gif\"></td>"
      show "</tr>"
    show "</form>"
  show "</table>"
}

do_Download()
{
  local file=$1
  # ------------------------
  local source=
  case "$file" in
    "callfile") source="$ASTERISK_FILE_CALLS";;
    "astdb") source="$ASTERISK_FILE_ASTDB";;
    *) source="";;
  esac
  if [ -n "$source" -a -f $source ]
  then
    http_header download "ctype=text/plain;filename=`echo \"$source\" | sed -n 's|.*/\(.*\)|\1|p'`"
    cat $source
  else
    show_html_header "$ASTERISK_PACKAGE $ASTERISK_GUI_VERSION"
    show_error "$ASTERISK_PACKAGE" "$_asterisk_error_downloadfilenotfound ($source)"
    show_html_footer
  fi
}

do_Action()
{
  local action=$1
  # ------------------------
  case "$action" in
    "overview"|"registry"|"peers"|"channels"|"calls")
      check_rights "$ASTERISK_RIGHTS_SECTION" "view";;
    "astdb"|"config"|"cli")
      check_rights "$ASTERISK_RIGHTS_SECTION" "admin";;
    *)
    ;;
  esac

  show_html_header "$ASTERISK_PACKAGE $ASTERISK_GUI_VERSION"
  _overview=$_asterisk_tab_overview
  _registry=$_asterist_tab_registry
  _peers=$_asterisk_tab_peers
  _channels=$_asterisk_tab_channels
  _calls=$_asterisk_tab_calls
  _astdb=$_asterisk_tab_astdb
  _config=$_asterisk_tab_config
  _cli=$_asterisk_tab_cli

  if [ -n "`check_UserRight admin`" ]
  then
    show_tab_header "$_overview" "$myname?action=overview" \
                    "$_registry" "$myname?action=registry" \
                    "$_peers" "$myname?action=peers" \
                    "$_channels" "$myname?action=channels" \
                    "$_calls" "$myname?action=calls&sfrom=`date +%Y-%m-%d`" \
                    "$_astdb" "$myname?action=astdb" \
                    "$_config" "$myname?action=config" \
                    "$_cli" "$myname?action=cli"
  else
    show_tab_header "$_overview" "$myname?action=overview" \
                    "$_registry" "$myname?action=registry" \
                    "$_peers" "$myname?action=peers" \
                    "$_channels" "$myname?action=channels" \
                    "$_calls" "$myname?action=calls&sfrom=`date +%Y-%m-%d`"
  fi

  case "$action" in
    "overview") show_Overview "$FORM_cmd";;
    "registry") for_all_channeltypes 'show_Registry "$type" "$description"; show "<br>"';;
    "peers")    for_all_channeltypes 'show_Peers "$type" "$description"; show "<br>"';;
    "channels") for_all_channeltypes 'show_Channels "$type" "$description"; show "<br>"';;
    "calls")    show_CallFile "$FORM_sfld" "$FORM_srdr" "$FORM_sfrom" "$FORM_sto" "$FORM_scaller" "$FORM_sname" "$FORM_scallee" "$FORM_sline" "$FORM_sufld";;
    "astdb")    show_AstDB "$FORM_family" "$FORM_key" "$FORM_data" "$FORM_cmd";;
    "config")   show_Config "$FORM_file" "$FORM_content";;
    "cli")      show_CLI "$FORM_cmd";;
    *)          show_error "$ASTERISK_PACKAGE" "$_asterisk_error_badaction ($action)";;
  esac
  show_html_footer
}

# ----------------------------------------------------------------------------

if [ -n "$FORM_download" ]
then
  do_Download "$FORM_download"
else
  if [ -n "$FORM_action" ]
  then
    do_Action "$FORM_action"
  else
    echo "ERROR: The GUI cannot be called from the command line!"
    errorlevel 1
  fi
fi
return $asteriskerrorlevel
